﻿using LuxSciWebApiWrapper.Exceptions;
using LuxSciWebApiWrapper.LuxApiHttpClient;
using LuxSciWebApiWrapper.Models;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading.Tasks;

namespace LuxSciWebApiWrapper
{
    public class UserApiWrapper
    {
        //members
        private string userRequestPath = string.Empty;
        private string userName = string.Empty;
        private string hostName = string.Empty;

        public UserApiWrapper()
        {
            //initialize members
            userRequestPath = ConfigurationManager.AppSettings["UserPath"];
            userName = ConfigurationManager.AppSettings["UserName"];
            hostName = ConfigurationManager.AppSettings["Host"];
        }

        //get user profile
        public async Task<UserProfile> GetUserProfile()
        {
            //get authentcation code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAuthenticationToken();
            LuxSciResponse luxSciResponse = null;

            //build signature and path
            string path = userRequestPath + "/"+userName + "/profile";
            string signature= AuthApiWrapper.GetHMACSignature(authCode + "\nGET\n" + path + "\n\n\n");

            //call httpclient
            using (UserHttpClient userHttpClient = new UserHttpClient())
            {
                luxSciResponse = await userHttpClient.GetUserProfile(path,authCode, signature);
            }

            //return user profile json data, if success is 1.
            if (luxSciResponse.success == 1)
            {
                return JsonConvert.DeserializeObject<UserProfile>(luxSciResponse.data.ToString());
            }
            else
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
        }

        //send mail
        public async Task<bool> SendMail(EmailRequest emailRequest)
        {

            //get authentication code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAuthenticationToken();
            LuxSciResponse luxSciResponse = null;

            //build path and signature
            string path = userRequestPath + "/" + userName + "/email/compose/secureline/send";
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nPOST\n" + path + "\n\n" + AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject(emailRequest).Trim()) + "\n");

            //call httpclient
            using (UserHttpClient userHttpClient = new UserHttpClient())
            {
                luxSciResponse = await userHttpClient.SendMail(path, authCode, signature,emailRequest);
            }

            //throw error, if the success is not 1
            if (luxSciResponse.success != 1)
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
            else
            {
                return true;
            }
        }

        #region send mail with attachment
        public async Task<bool> SendMailWithAttachment(EmailRequest emailRequest, string filename)
        {
            //get authentication code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAuthenticationToken();
            LuxSciResponse luxSciResponse = null;
            string boundary = Guid.NewGuid().ToString();

            //get file as byte array
            byte[] fileContents = File.ReadAllBytes(filename);

            //attach the file to the emailrequest
            AttachFile(emailRequest, filename, fileContents);

            //get file and email content
            var content = GetMultipartFormContent(emailRequest, filename, boundary, fileContents);

            //build path and signature
            string path = userRequestPath + "/" + userName + "/email/compose/secureline/send";
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nPOST\n" + path + "\n\n" + AuthApiWrapper.ComputeSha256Hash(JsonConvert.SerializeObject(emailRequest).Trim()) + "\n");

            //call httpclient
            using (UserHttpClient userHttpClient = new UserHttpClient())
            {

                var request = new HttpRequestMessage(HttpMethod.Post, new Uri($"https://{hostName}/{path}"))
                {
                    Version = HttpVersion.Version10,
                    Content = content
                };


                luxSciResponse = await userHttpClient.SendMailWithAttachment(path, authCode, signature, request, boundary);
            }

            //throw error, if the success is not 1
            if (luxSciResponse.success != 1)
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
            else
            {
                return true;
            }
        }

        //attach file to the email request
        private void AttachFile(EmailRequest emailRequest, string filename, byte[] fileContents)
        {
            List<Attachment> attachments = new List<Attachment>();
            Attachment attachement = new Attachment();
            attachement.Name = Path.GetFileName(filename);
            attachement.Hash = AuthApiWrapper.ComputeSha256Hash(fileContents);
            attachments.Add(attachement);
            emailRequest.Attachments = attachments.ToArray();
        }

        //get file byte array
        private ByteArrayContent GetFileByteArrayContent(string filename, byte[] fileContents)
        {
            

            ByteArrayContent fileByteArrayContent = new ByteArrayContent(fileContents);
            fileByteArrayContent.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            fileByteArrayContent.Headers.ContentDisposition.Name = "\"files\"";
            fileByteArrayContent.Headers.ContentDisposition.FileName = "\"" + Path.GetFileName(filename) + "\"";
            fileByteArrayContent.Headers.ContentType = new MediaTypeHeaderValue("application/octet-stream");

            return fileByteArrayContent;
        }

        //get mail byte array
        private ByteArrayContent GetMailByteArrayContent(EmailRequest emailRequest)
        {
            ByteArrayContent mailByteArrayJson = new ByteArrayContent(Encoding.UTF8.GetBytes(JsonConvert.SerializeObject(emailRequest).Trim()));

            mailByteArrayJson.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
            mailByteArrayJson.Headers.ContentDisposition = new ContentDispositionHeaderValue("form-data");
            mailByteArrayJson.Headers.ContentDisposition.Name = "\"json\"";
            mailByteArrayJson.Headers.ContentDisposition.FileName = "\"json.js\"";

            return mailByteArrayJson;
        }

        //compose mail byte array and file byte array into content
        private MultipartFormDataContent GetMultipartFormContent(EmailRequest emailRequest,string filename,string boundary, byte[] fileContents)
        {
            var content = new MultipartFormDataContent(boundary);
            content.Headers.Remove("Content-Type");
            content.Headers.TryAddWithoutValidation("Content-Type", "multipart/form-data; boundary=" + boundary);
            content.Add(GetMailByteArrayContent(emailRequest));
            content.Add(GetFileByteArrayContent(filename,fileContents));

            return content;
        }
        #endregion

        public async Task<string> GetMailSentReport()
        {
            //get authentication code
            AuthApiWrapper authApi = new AuthApiWrapper();
            string authCode = await authApi.GetAuthenticationToken();
            LuxSciResponse luxSciResponse = null;

            //build path and signature
            string path = userRequestPath + "/" + userName + "/report/email/webmail/sent";
            string signature = AuthApiWrapper.GetHMACSignature(authCode + "\nGET\n" + path + "\n\n\n");

            //call HttpClient
            using (UserHttpClient userHttpClient = new UserHttpClient())
            {
                luxSciResponse = await userHttpClient.GetEmailSentReport(path, authCode, signature);
            }

            //throw exception, if the success is not 1
            if (luxSciResponse.success != 1)
            {
                throw new ApiCallFailedException(luxSciResponse.error_message);
            }
            else
            {
                return luxSciResponse.data.ToString();
            }
        }

    }
}
